_DrawLight_OMNI.vsh
{
	include generic_vertex.vsh
}

_DrawLight_OMNI.fsh
{
	uniform sampler2DRect	u_NormalBuffer;
	uniform sampler2DRect	u_SpecularBuffer;
	uniform sampler2DRect	u_DepthBuffer;
	uniform sampler2DRect	u_ColorBuffer;
#if defined(Stages1)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform vec2		u_AttenuationSizes;
	uniform vec4		u_AttenuationColor;
#endif
#if defined(Stages2)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform vec2		u_Blend;
	uniform vec2		u_AttenuationSizes[2];
	uniform vec4		u_AttenuationColor[2];
#endif
#if defined(Stages3)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform sampler2DRect	u_AttenuationMap2;
	uniform vec2		u_Blend[2];
	uniform vec2		u_AttenuationSizes[3];
	uniform vec4		u_AttenuationColor[3];
#endif
#if defined(Stages4)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform sampler2DRect	u_AttenuationMap2;
	uniform sampler2DRect	u_AttenuationMap3;
	uniform vec2		u_Blend[3];
	uniform vec2		u_AttenuationSizes[4];
	uniform vec4		u_AttenuationColor[4];
#endif
	uniform vec3		u_LightOrigin;
	uniform vec3		u_ViewOrigin;
	uniform vec3		u_LightColor;
	uniform mat4		u_LightAttenuationMatrix;
	uniform mat4		u_UnprojectMatrix;
	uniform float		u_LightRadius;
	uniform float		u_noSpecular;
#ifndef Stages1
	include Blend.inc
#endif
	out	vec3	fragData;

	include pbr.inc
#if defined(FAKE_PPL)
	include UnPackNormal.inc
#endif

	void main(void)
	{
		vec4 specular = texture(u_SpecularBuffer, gl_FragCoord.xy);
		if (specular.a == 1.0)
		{
			discard;	// no portal surfaces draw
			return;
		}

		// reconstruct vertex position in world space
		float depth = texture(u_DepthBuffer, gl_FragCoord.xy).r;
		vec4 P = u_UnprojectMatrix * vec4(gl_FragCoord.xy, depth, 1.0);
		P.xyz /= P.w;

		// fast culling
		float dist = distance(u_LightOrigin, P.xyz);
		if (dist > u_LightRadius)
		{
			discard;
			return;
		}

		// compute color attenuation
		vec3 texAtten = (u_LightAttenuationMatrix * vec4(P.xyz, 1.0)).xyz;
		texAtten.z = abs(texAtten.z - 0.5);
		if (texAtten.z >= 0.5)
		{
			discard;
			return;
		}

		vec3 lit = 2.0 * (0.5 - texAtten.z) * u_LightColor;

#if defined(Stages1)
		lit *= (texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes) * u_AttenuationColor).rgb;
#endif
#if defined(Stages2)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		lit *= (Blend(s,d,u_Blend)).rgb;
#endif
#if defined(Stages3)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		d = Blend(s,d,u_Blend[0]);
		s = texture(u_AttenuationMap2, texAtten.xy * u_AttenuationSizes[2]) * u_AttenuationColor[2];
		lit *= (Blend(s,d,u_Blend[1])).rgb;
#endif
#if defined(Stages4)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		d = Blend(s,d,u_Blend[0]);
		s = texture(u_AttenuationMap2, texAtten.xy * u_AttenuationSizes[2]) * u_AttenuationColor[2];
		d = Blend(s,d,u_Blend[1]);
		s = texture(u_AttenuationMap3, texAtten.xy * u_AttenuationSizes[3]) * u_AttenuationColor[3];
		lit *= (Blend(s,d,u_Blend[2])).rgb;
#endif
		if (dot(lit,lit) < 0.00001526)	// instead: if (length(lit) < 1.0/256.0)
		{
			discard;	// attenuated color is too dark
			return;
		}

		vec4 sN = texture(u_NormalBuffer, gl_FragCoord.xy);
#if defined(FAKE_PPL)
		vec3 N = UnPackNormal(sN.xy);
#else
		vec3 N = sN.xyz;
#endif

		// compute light direction in world space
		vec3 L = (u_LightOrigin - P.xyz) / dist;

		float NL = dot(N, L);
		if (NL <= 0.0)		// back face
		{
			discard;
			return;
		}
		
		lit *= NL;

		// compute view direction in world space
		vec3 V = normalize(u_ViewOrigin - P.xyz);

		DoPBR(lit, specular, N, L, V, NL, sN.a);
	}
}


_DrawLight_PROJ.vsh
{
	include generic_vertex.vsh
}

_DrawLight_PROJ.fsh
{
	uniform sampler2DRect	u_NormalBuffer;
	uniform sampler2DRect	u_SpecularBuffer;
	uniform sampler2DRect	u_DepthBuffer;
	uniform sampler2DRect	u_ColorBuffer;
#if defined(Stages1)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform vec2		u_AttenuationSizes;
	uniform vec4		u_AttenuationColor;
#endif
#if defined(Stages2)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform vec2		u_Blend;
	uniform vec2		u_AttenuationSizes[2];
	uniform vec4		u_AttenuationColor[2];
#endif
#if defined(Stages3)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform sampler2DRect	u_AttenuationMap2;
	uniform vec2		u_Blend[2];
	uniform vec2		u_AttenuationSizes[3];
	uniform vec4		u_AttenuationColor[3];
#endif
#if defined(Stages4)
	uniform sampler2DRect	u_AttenuationMap0;
	uniform sampler2DRect	u_AttenuationMap1;
	uniform sampler2DRect	u_AttenuationMap2;
	uniform sampler2DRect	u_AttenuationMap3;
	uniform vec2		u_Blend[3];
	uniform vec2		u_AttenuationSizes[4];
	uniform vec4		u_AttenuationColor[4];
#endif
	uniform vec3		u_LightOrigin;
	uniform vec3		u_ViewOrigin;
	uniform vec3		u_LightColor;
	uniform mat4		u_LightAttenuationMatrix;
	uniform mat4		u_UnprojectMatrix;
	uniform float		u_LightRadius;
	uniform float		u_noSpecular;
#ifndef Stages1
	include Blend.inc
#endif
	out	vec3	fragData;

	include pbr.inc
#if defined(FAKE_PPL)
	include UnPackNormal.inc
#endif

	void main(void)
	{
		vec4 specular = texture(u_SpecularBuffer, gl_FragCoord.xy);
		if (specular.a == 1.0)
		{
			discard;	// no portal surfaces draw
			return;
		}

		// reconstruct vertex position in world space
		float depth = texture(u_DepthBuffer, gl_FragCoord.xy).r;
		vec4 P = u_UnprojectMatrix * vec4(gl_FragCoord.xy, depth, 1.0);
		P.xyz /= P.w;

		// fast culling
		float dist = distance(u_LightOrigin, P.xyz);
		if (dist > u_LightRadius)
		{
			discard;
			return;
		}

		// compute color attenuation
		vec4 texAtten = u_LightAttenuationMatrix * vec4(P.xyz, 1.0);
		if(texAtten.w <= 0.0)
		{
			discard;	// fragment is behind the near clip plane
			return;
		}
		
		vec3 lit = (1.0 - texAtten.z) * u_LightColor;
		texAtten.xy /= texAtten.w;

#if defined(Stages1)
		lit *= (texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes) * u_AttenuationColor).rgb;
#endif
#if defined(Stages2)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		lit *= (Blend(s,d,u_Blend)).rgb;
#endif
#if defined(Stages3)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		d = Blend(s,d,u_Blend[0]);
		s = texture(u_AttenuationMap2, texAtten.xy * u_AttenuationSizes[2]) * u_AttenuationColor[2];
		lit *= (Blend(s,d,u_Blend[1])).rgb;
#endif
#if defined(Stages4)
		vec4 d = texture(u_AttenuationMap0, texAtten.xy * u_AttenuationSizes[0]) * u_AttenuationColor[0];
		vec4 s = texture(u_AttenuationMap1, texAtten.xy * u_AttenuationSizes[1]) * u_AttenuationColor[1];
		d = Blend(s,d,u_Blend[0]);
		s = texture(u_AttenuationMap2, texAtten.xy * u_AttenuationSizes[2]) * u_AttenuationColor[2];
		d = Blend(s,d,u_Blend[1]);
		s = texture(u_AttenuationMap3, texAtten.xy * u_AttenuationSizes[3]) * u_AttenuationColor[3];
		lit *= (Blend(s,d,u_Blend[2])).rgb;
#endif
		if (dot(lit,lit) < 0.00001526)	// instead: if (length(lit) < 1.0/256.0)
		{
			discard;	// attenuated color is too dark
			return;
		}

		vec4 sN = texture(u_NormalBuffer, gl_FragCoord.xy);
#if defined(FAKE_PPL)
		vec3 N = UnPackNormal(sN.xy);
#else
		vec3 N = sN.xyz;
#endif

		// compute light direction in world space
		vec3 L = (u_LightOrigin - P.xyz) / dist;

		float NL = dot(N, L);
		if (NL <= 0.0)		// back face
		{
			discard;
			return;
		}

		lit *= NL;

		// compute view direction in world space
		vec3 V = normalize(u_ViewOrigin - P.xyz);

		DoPBR(lit, specular, N, L, V, NL, sN.a);
	}
}


_DrawLight_PARALLEL.vsh
{
	include generic_vertex.vsh
}

_DrawLight_PARALLEL.fsh
{
	uniform sampler2DRect	u_NormalBuffer;
	uniform sampler2DRect	u_SpecularBuffer;
	uniform sampler2DRect	u_DepthBuffer;
	uniform sampler2DRect	u_ColorBuffer;
	uniform vec3		u_ViewOrigin;
	uniform vec3		u_LightOrigin;
	uniform vec3		u_LightColor;
	uniform mat4		u_UnprojectMatrix;
	uniform float		u_noSpecular;
	out	vec3	fragData;

	include pbr.inc

	void main(void)
	{
		vec4 specular = texture(u_SpecularBuffer, gl_FragCoord.xy);
		if (specular.a == 1.0)
		{
			discard;	// no portal surfaces draw
			return;
		}

		// reconstruct vertex position in world space
		float depth = texture(u_DepthBuffer, gl_FragCoord.xy).r;
		vec4 P = u_UnprojectMatrix * vec4(gl_FragCoord.xy, depth, 1.0);
		P.xyz /= P.w;

		vec4 sN = texture(u_NormalBuffer, gl_FragCoord.xy);
		vec3 N = sN.xyz;

		// compute light direction in world space
		float dist = distance(u_LightOrigin, P.xyz);
		vec3 L = (u_LightOrigin - P.xyz) / dist;

		float NL = dot(N, L);
		if (NL <= 0.0)		// back face
		{
			fragData = vec3(0.0);
			return;
		}

		vec3 lit = u_LightColor * NL;

		// compute view direction in world space
		vec3 V = normalize(u_ViewOrigin - P.xyz);

		DoPBR(lit, specular, N, L, V, NL, sN.a);
	}
}
